;*********************************************************************************************************
;***************  digitales Zhlwerk TapeTimer.asm  ******************************************************
;***************  www.stefankneller.de								**********
;*********************************************************************************************************
;**********											**********
;**********  Projekt:	elektronisches Zhlwerk fr Kassettenrekorder mit 7-Segmentanzeige	**********
;**********											**********
;**********  Aufgabe:	Messen der Banddicke							**********
;**********		Ausgabe eines Stoppsignals bei Zhlwerkstellung 0000			**********
;**********		Anzeige von:								**********
;**********		- absoluter Bandposition in (Stunden) Minuten und Sekunden		**********
;**********		  im Aufnahme-/Wiedergabe- und Spulmodus				**********
;**********		- Umdrehungen des Wickeltellers						**********
;**********		- Banddicke in m							**********
;**********		- Kopfbetriebsstunden im Aufnahme- oder Wiedergabemodus			**********
;**********											**********
;**********  Beschreibung:									**********
;**********  Die Anwendung geht von einer konstanten Bandgeschwindigkeit von 4,75 cm/Sek aus.	**********
;**********  Die Bandgeschwindigkeit wird hier nicht gemessen und abweichende Werte fhren zu	**********
;**********  einer ungenauen Anzeige.
;**********  
;**********  Das Rechenwerk zhlt mittels zweier versetzt angeordneter Lichtschranken
;**********  und einem Schwarzweimuster auf der Aufwickeltrommel Anzahl, Richtung und
;**********  Drehzahl der Umdrehungen des Tellers.
;**********  Die Anzeige "U" fr Umdrehungen gibt die Anzahl der Tellerumdrehungen an.
;**********  Die Anzeige ist an jeder Stelle mit einem langen Tastendruck rcksetzbar
;**********  und steuert bei Erreichen der Zhlwerkstellung "0000" den Nullstoppausgang an.
;**********  Wenn der Kassettenrekorder ber eine entsprechnede Funktion verfgt, kann diese
;**********  hierduch angesteuert werden.
;**********  berlauf des Umdrehungszhlwerkes ist bei 2000.
;**********  
;**********  Wenn der Rekorder im Wiedergabe- oder Aufnahmemodus ist wird aus der im
;**********  Mittelwertspeicher gegltteten Tellerdrehzahl und der Tellerdrehzahl von einigen
;**********  (hier 8) Umdrehungen vorher ein Dickenwert fr die Banddicke berechnet.
;**********  die Varianz der Einzelimpulse bei Wiedergabe betrgt hier ca. 1,43% des Mittelwertes
;**********  Dieser Wert ist mit Schwankungen behaftet und wird daher durch ein in der Tiefe
;**********  umschaltbares Filter U beruhigt.
;**********  Drei unterschiedlich starke Filterungen werden whrend der Synchronisierphase angewand.
;**********  Der Dickenwert mit dem intern gerechnet wird entspricht der Banddicke in Millimeter/100000
;**********  Die Anzeige "" zeigt die ermittelte Banddicke in m an.
;**********  Die Anzeige "r" zeigt nach Synchronisation den ermittelten Bandwickelradius in mm.
;**********  
;**********  Anhand des Dickenwertes und der aktuellen Tellerdrehzahl wird die Zahl der
;**********  aufgewickelten Bandlagen und daraus die abgelaufene Spielzeit ermittelt.
;**********  Die Anzeige "L" zeigt die ermittelten Bandlagen, Wertebereich 0 bis 2000 Lagen.
;**********  Die Anzeige "S" zeigt nach Synchronisation die ermittelte abgelaufene Spielzeit
;**********  in vollen Sekunden.
;**********  
;**********  Wenn alle Werte innerhalb ihrer Grenzen und stabil sind, nicht aber vor Beginn
;**********  des Synchronisierfensters (hier 30 Sekunden) werden sie ins Display bernommen,
;**********  das Flag "Synchron" gesetzt und die quarzgefhrte Zeit im Display nur noch
;**********  schrittweise an die errechneten Zeiten nachgefhrt. (Ziehrate 1 Frame/Sekunde)
;**********  Am Ende des Synchronisierfensters (hier nach 90 Sekunden) wird die Synchronisierung
;**********  erzwungen.
;**********  Der synchrone Zustand wird durch den Dezimalpunkt auf der letzten Stelle der Anzeige angezeigt
;**********  Die Anzeige "T" zeigt die abgelaufen Spielzeit 5-stellig in H:MM:SS oder 4-stellig in MM:SS
;**********  
;**********  Bis die initiale Messphase abgeschlossen ist wird im Display:
;**********  (Prprozessoranweisung: "Messanzeige")
;**********  - "CALC" blinkend, Tastenkommandos gesperrt oder
;**********  - die aus den bis dahin bekannten Werten errechenbaren Daten gezeigt
;**********  
;**********  Im Aufnahme- und Wiedergabebetrieb wird alle 10 Sekunden der Kopfstundenzhler
;**********  um ein Bit erhht. Seine Tiefe von 24 Bit reicht fr 2^24/360 Sekunden = 46603 Stunden.
;**********  Die Anzeige "H" zeigt die vollen Kopfstunden  an.
;**********  
;**********  Kurz vor dem Wegfall der Versorgungsspannung werden die Werte fr Zhlwerksstand,
;**********  Banddicke und Kopfstunden ins EEPROM gerettet.
;**********  
;**********  Nach dem Einschalten werden die Filter U (Banddickenwert), der Betriebsstundenzhler
;**********  und der Umdrehungszhler mit den geretteten Werten aus dem EEPROM befllt.
;**********  Ausschalten bei gedrckter Taste lscht den Betriebsstundenzhler.
;**********  
;**********  Das Display zeigt die abgelaufen Spielzeit "T" an.
;**********  Ein kurzer Tastendruck bringt rotierend weitere Messwerte auf die Anzeige.
;**********  Umdrehungen "U", Banddicke "", Stunden "H", Wickelradius "r", Lagen "L",
;**********  abgelaufene Sekunden "S" oder einen Lauftext.
;**********  
;**********  Bei stehendem Laufwerk wird die errechnete Lagenzahl auf denjenigen Wert gezwungen,
;**********  der sich aus Quarzzeitanzeige und Banddicke ergibt. Damit wird ein Springen der
;**********  Zeitanzeige zum Spulbeginn unterbunden.
;**********  
;**********  Das ffnen des Kassettenfaches setzt die Werte abgelaufene Zeit "T" und "S", die
;**********  Bandlagen "L", sowie die Flags Capture-, Segmentlaufzeit-, Banddicke-valide und
;**********  Synchron zurck.
;**********  
;**********  Die Funktion "Einschalten mit gedrckter Taste" ist noch frei.
;**********  
;**********  bekannte Schwchen:
;**********  Da die Drehzahlimpulse immer schwanken werden sie ber mehrere volle Umdrehungen
;**********  (Mittelwerttiefe/zebra) arithmetisch gemittelt.
;**********  Da die Drehzahlabnahme aber nicht linear erfolgt ergibt sich dadurch ein Fehler.
;**********  Er ist umso kleiner je kleiner die Mittelwerttiefe ist.
;**********  Zwischen Wiedergabe und Spulen muss eine minimale Bedienpause eingehalten werden,
;**********  sonst bleibt das Play-Flag gesetzt, weil der Laufwerksherzschlag nicht so schnell
;**********  bis auf 0 runtergezhlt wird.
;**********  Wenn der Umdrehungszhler vorwrts und rckwrts unterschiedlich zhlt oder
;**********  hakelig luft ist eine mechanische/elektrische Justage der Lichtschranken ntig.
;**********  Versuche mit einem von der Bandposition abhngigen Zeitfenster fr eine
;**********  Zwangssynchronisierung waren erfolglos, da unprzise Kassetten lange Sync-Zeiten bentigen knnen
;**********  Im Spulmodus mit etwas lockererem Bandwickel wird keine angepasste Banddicke fr
;**********  die Berechnungen angewendet, da lockere Bandwickel nur einen sehr kleinen Fehler verursachen.
;**********  
;**********  Bedingungen die zu Anzeigefehlern fhren (quantitative Fehleranalyse nicht durchgefhrt):
;**********  - 	wahrer Wickelkernradius ungleich Defaultwert
;**********  -  Beschrnkung auf die Abtastung eines Wickeltellers
;**********  - 	Bandgeschwindigkeit schwankt und/oder ist ungleich 4,75 cm/s
;**********  -  arithmetisch gemittelte Drehzahlimpulse entsprechen nicht exakt dem wahren Mittel
;**********  - 	unterschiedlich straff gewickelter Bandwickel
;**********  - 	Fehler durch Rundungen und Einschrnkungen von Wertebereichen
;**********  - 	Controllertakt ungleich 16,000 MHz
;**********  
;*********************************************************************************************************
;**********  Hardware:										**********
;**********		Controller ATMEGA 8 mit 4 MHz Quarz					**********
;**********		2 Reflexlichtschranken am Aufwickelteller				**********
;**********		eine Gabellichtschranke  am Kassettenfach				**********
;**********		3 Komparatoren mit Hysterese zur Aufbereitung Lichtschrankensignale	**********
;**********		ein Taster								**********
;**********		ein 4 oder 5-stelliges 7-Segment LED-Display				**********
;**********		eigene Spannungsversorgung aus 12 Volt~					**********
;**********											**********
;*********************************************************************************************************
;**********											**********
;**********		Idee, Hardware, Software:	Stefan Kneller				**********
;**********		mit Anregungen aus der EU-Patentanmeldung EP1246189A2			**********
;**********											**********
;*********************************************************************************************************
.include"m8def.inc"
;*********************************************************************************************************
;**********  ASCII Klartext fr das Ende des HEX-Files						**********
;*********************************************************************************************************
.equ	Kommentarzeilen	= 8
.org flashend-(8*Kommentarzeilen-1)	;damit endet der Kommentar genau im letzten Flashbereich

.db	"   Tapetimer    "
.db	"Softwareversion "
.db	"0.1 fr ATMEGA8 "
.db	"  FuseH = 0xC9  "
.db	"  FuseL = 0XFF  "
.db	"Letzte  nderung"
.db	"   04.10.2014   "
.db	" Stefan Kneller "

;*********************************************************************************************************
;**********  Todo:	
;**********		- "CALC" blinkt unsauber und blitz zwischen Spulen und Stopp auf
;**********		
;**********		- reduzierten Bobbyradius verwerfen oder aktivieren und testen
;**********		
;**********		- einige Befehlszeilen haben noch keinen aussagekrftigen Kommentar
;**********		
;**********		- Anzeige mit gem. Anode testen
;**********		
;*********************************************************************************************************

;*********************************************************************************************************
;***************  an beliebig ausgewhlten Kassetten ermittelte Werte				**********
;*********************************************************************************************************
;Filtertiefe 16/Offset = 0    	Wickelradius	Lagen	Dicke m	Dicke m	Differenz
;				Millimeter		gemessen	Rekorder	%	absolut
;TDK SA-X    C90		25,75		1172	12,58		12,34		-1,9	-0,24
;SONY UX-S   C90          	25,10		1166	12,09		12,06		-0,2	-0,03
;FUJI KII    C90 o. Filz 	25,10		800	17,30		17,37		 0,4	 0,07
;FUJI DRII   C60 o. Filz 	25,25		799	17,85		17,44		-2,3	-0,41
;TDK MA-G    C60		24,10		820	15,97		15,86		-0,7	-0,11
;SONY EX-F   C60		23,75		814	15,66		15,36		-1,9	-0,30
;NONAME      C60		25,00		792	17,68		17,42		-1,9	-0,24

;*********************************************************************************************************
;***************  Formelsammlung								**********
;*********************************************************************************************************
;***************  Counter1 misst die Dauer jedes Wei-Schwarzfeld-Vorbeilaufs an Geber A	**********
;***************  Grundformel:	counts = r * 2 * Pi * CLK/(Vo * Zebra * Teiler)			**********
;***************	 counts = Zhlerstand Timer nach einem Wei-Schwarz-Feld-Wechsel	**********
;***************  	      r = Wickelradius in mm						**********
;***************	     Pi = 3,14159							**********
;***************	    CLK = 4.000.000 Hz							**********
;***************	     Vo = Bandgeschwindigkeit in mm/s, hier 47,625			**********
;***************          Zebra = Anzahl weier Felder auf dem Radumfang, hier 8		**********
;***************	 Teiler = Vorteiler 16-Bit Counter, hier 64				**********
;***************             Ro = Wickelkernradius in mm					**********
;***************              d = Banddicke in mm						**********
;***************         T_segm = Zhlerstand A Counter 1					**********
;***************   T_Segm_delta = Zhlerstand B Counter 1 vor delta_v Umdrehungen		**********
;***************        delta_v = Umdrehungszahl zwischen den Zhlerstnden A und B		**********
;***************              T = abgelaufene Zeit in Sekunden					**********
;***************  	  bei r = 11mm ergeben sich 11367 counts				**********
;***************	  bei r = 28mm ergeben sich 28935 counts				**********
;***************										**********
;*********************************************************************************************************
;***************  Banddicke:									**********
;***************	d = (T_segm-T_segm_delta_v)*Vo*Zebra*Teiler1/2*Pi*Clk*delta_v		**********
;***************										**********
;*********************************************************************************************************
;***************  Bandlagen: (aus Segmentlaufzeit und Banddicke)				**********
;***************  Grundformel: counts = r * 2 * Pi * CLK/(Vo * zebra * Vorteiler1)		**********
;***************  Weiter gilt: Lagen = (Radius_aktuell - Ro)/Banddicke				**********
;***************  beide Gleichungen zusammgefhrt:						**********
;***************        Lagen = (counts_aktuell*Vo*Zebra*Teiler1/(2*Pi*CLK))-Ro)/Banddicke	**********
;***************										**********
;*********************************************************************************************************
;***************  Bandlagen: (aus abgelaufenen Sekunden und Banddicke)				**********
;***************  L = (Wurzel((Banddicke/2+Ro)^2+T*Banddicke*Vo/Pi)-Banddicke/2-Ro)/Banddicke	**********
;***************										**********
;*********************************************************************************************************
;***************  abgelaufenen Sekunden:							**********
;***************	T = (2 * PI * Lagen * (Ro + (Lagen + 1) * (Banddicke / 2))) / Vo	**********
;***************										**********
;*********************************************************************************************************
;***************  Wickelradius:									**********
;***************		r = counts * Vo * Zebra * Teiler1/(2 * Pi * CLK)		**********
;***************                r = Ro + Lagen * d						**********
;***************										**********
;*********************************************************************************************************

;*********************************************************************************************************
;***************  Definition fester Werte							**********
;*********************************************************************************************************
.equ	Bandgeschw	= 47625		;Bandgeschwindigkeit in m/s (Wertebereich: 1 bis 1005322/zebra?)
.equ	Wickelkern	= 11100		;Radius des leeren Wickeltellers in m
.equ	Zebra		= 8		;Anzahl der Lichtschranken-Hell-Felder auf dem Wickeltellerumfang
					;Wertebereich: 4 und 8
.equ	Max_Play	= (wickelkern*39269908)/(Bandgeschw*Zebra*107);107% von 4,75cm/s, hchste erlaubte Drehzahl
					;um als Wiedergabe zu gelten; K = 200*Pi*CLK/Vorteiler1
.equ	Min_Dicke	= 8000		;kleinste erlaubte Banddicke in nm
.equ	Max_Dicke	= 21000		;grte erlaubte Banddicke in nm
.equ	Max_Lagen	= 2000		;grte erlaubte Anzahl von Bandlagen
.equ	Max_Umdrehung	= 2000		;grter erlaubter Anzeigewert des Umdrehungszhlwerkes
.equ	Helligkeit	= 18		;Displayhelligkeit, Wertebereich 1% - 100%
.equ	Kennung		= 25		;Dauer der Einblendung Kennbuchstabe in Frames
.equ	Mittelwerttiefe	= 4		;Anzahl Umdrehungen fr die Mittelwertbildung V und "Delta V"
					;Wertebereich 2,4,8,16
.equ	Delta_V		= 4		;Banddickenmessung anhand Wickelzunahme nach N Umdrehungen
					;Wertebereich: 2,3,4,5 usw. bis 32 (weiter nicht ermittelt)
					;effektive Kombination: mittelwerttiefe = Delta_v
.equ	Delta_U		= 100		;hchste erlaubte nderungsrate Dickenwert*100/Sekunde fr Synchronisierung
.equ	Kopfintervall	= 10		;Zeitintervall fr Kopfstundenzhlung in Sekunden, Wertebereich: 0 < x < 57
.equ	Laufwerkpuls	= (24/zebra)+2	;erlaubte Zeit ohne Gebersignal fr "Laufwerksbewegung vorhanden" in Frames
					;bei unsymmetrischer Lichtschrankenanordnung mehr!
.equ	Frameoffset	= 10		;der Framezhler des Uhrwerkes zhlt von Offset bis (Offset + 25)
.equ	Sync_sperre	= 30		;die ersten N Sekunden ist die Synchronisierung gesperrt
.equ	Sync_pflicht	= 90		;N Sekunden nach Messbeginn wird zwangssynchronisiert
.equ	Filterwechsel_U	= 50		;nach N Sekunden Messdauer wird von 1ter auf 2te Filterkonstante geschaltet
.equ	Filtertiefe_U1	= 32		;1te Filtertiefe bis Filterwechsel, Wertebereich: 16,32,64,128
.equ	Filtertiefe_U2	= 128		;2te Filtertiefe ab Filterwechsel, Wertebereich: 16,32,64,128
.equ	Filtertiefe_U3	= 512		;3te Filtertiefe nach Synchronisierung, Wertebereich: 256, 512
.equ	Ziehrate	= 1		;Zeit, die das Rechenwerk die Quarzzeit pro Sekunde ziehen kann in Frames
.equ	Dickenoffset	= 0		;Offset zur errechneten Dicke in 0,01m zwecks Fehlerreduzierung
.equ	Def_Stunden	= 1015		;Default-Wert Kopfstunden und
.equ	Def_Dicke	= 15000		;Default-Wert Dickenwert in nm
.equ	Modi		= 16		;Anzahl der Anzeigemodi
.equ	Anzeigeart	= 0		;Anzeigeverdrahtung: 0 = gemeinsame Kathode / 1 = gemeinsame Anode
.equ	Anzeigestellen	= 4		;wieviele Stellen hat die LED-Anzeige? Wertebereich: 4 und 5
.equ	Startmodus	= 0		;Anzeigeart nach dem Einschalten
.equ	Nullunterdrueck = 0		;0 = Unterdrckung fhrender Nullen inaktiv / 1 = aktiv
.equ	MessAnzeige	= 0		;Anzeige whrend der initialen Messphase: 0=Daten / 1="CALC" 
.equ	Textfeld	= string_hallo	;Textmeldung als Laufband in der Anzeige
.equ	Adr_eeprom	= 2		;unterstes Speicherziel im EEPROM fr zu rettenden Werte

;*********************************************************************************************************
;***************  Pinbelegung									**********
;*********************************************************************************************************
;                           _______
;                        1 /       |28
; Reset            C6 o---|        |---o   C5 Ausgang Kathodentreiber Digit 5
; Segmentausgang   D0 o---|        |---o   C4 Ausgang Kathodentreiber Digit 4
; Segmentausgang   D1 o---|        |---o   C3 Ausgang Kathodentreiber Digit 3
; Segmentausgang   D2 o---|        |---o   C2 Ausgang Kathodentreiber Digit 2
; Segmentausgang   D3 o---|        |---o   C1 Ausgang Kathodentreiber Digit 1
; Segmentausgang   D4 o---| ATMEGA |---o   C0 Ausgang "Nullstopp" und "Debug-LED"
;                 VCC o---|    8   |---o   GND
;                 GND o---|        |---o   AREF
;                XTAL o---|        |---o   AVCC
;                XTAL o---|        |---o   B5 Ausgang "Lichtschranken-LEDs"
; Segmentausgang   D5 o---|        |---o   B4 Eingang "Versorgungsspannung vorhanden"
; Segmentausgang   D6 o---|        |---o   B3 Eingang Taster
; Segmentausgang   D7 o---|        |---o   B2 Gebereingang C
; Gebereingang A   B0 o---|________|---o   B1 Gebereingang B

;*********************************************************************************************************
;***************  Verdrahtung Port B								**********
;*********************************************************************************************************
;PORT B
;BIT    7         6         5         4         3         2         1         0
;      NC        NC      IR-LED    Spannung   Taster    Geber C   Geber B   Geber A

.equ	A_LED		= 5		;Schaltausgang LED			}low-aktiv
.equ	A_Stop		= 5		;Schaltausgang Nullstopp		}high-aktiv
.equ	E_Spannung	= 4		;Eingang Versorgungsspannungsberwachung
.equ	E_Taster	= 3		;Eingang Taster, low-aktiv
.equ	E_Tape_out	= 2		;Eingang Lichtschranke C vom Kassettenschacht
.equ	E_Geber_B 	= 1		;Eingang Lichtschranke B vom Segmentrad
.equ	E_Geber_A 	= 0		;Eingang Lichtschranke A vom Segmentrad

;*********************************************************************************************************
;***************  Verdrahtung Port C Ausgnge							**********
;*********************************************************************************************************
;PORT C
;BIT    7         6         5         4         3         2         1         0
;     nicht     Reset    Stelle 5  Stelle 4  Stelle 3  Stelle 2  Stelle 1  Nullstop
;   vorhanden              LSB                                     MSB     und  LED

.equ	A_Digit5	= 5		;Ausgang Kathodentreiber Digit 5 LSB	}
.equ	A_Digit4	= 4		;Ausgang Kathodentreiber Digit 4	}
.equ	A_Digit3	= 3		;Ausgang Kathodentreiber Digit 3	}high-aktiv
.equ	A_Digit2	= 2		;Ausgang Kathodentreiber Digit 2	}
.equ	A_Digit1	= 1		;Ausgang Kathodentreiber Digit 1 MSB	}
.equ	A_IR_LED	= 0		;Ausgang IR-LEDs aller Lichtschranken	}low aktiv

;*********************************************************************************************************
;***************  Verdrahtung Port D								**********
;*********************************************************************************************************
;PORT D
;BIT    7         6         5         4         3         2         1         0
;Segm   G         dp        C         D         E         A         F         B

.equ	Seg_g		= 7		;7
.equ	Seg_dp		= 6		;6
.equ	Seg_c		= 5		;5
.equ	Seg_d		= 4		;4
.equ	Seg_e		= 3		;3
.equ	Seg_a		= 2		;2
.equ	Seg_f		= 1		;1
.equ	Seg_b		= 0		;0

;*********************************************************************************************************
;***************  Definition der Leuchtzeichen							**********
;*********************************************************************************************************
;Ziffern
.equ	Zeichen_0	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)
.equ	Zeichen_1	=            (1<<Seg_b)|(1<<Seg_c)
.equ	Zeichen_2	= (1<<Seg_a)|(1<<Seg_b)|           (1<<Seg_d)|(1<<Seg_e)|           (1<<Seg_g)
.equ	Zeichen_3	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)|                      (1<<Seg_g)
.equ	Zeichen_4	=            (1<<Seg_b)|(1<<Seg_c)|                      (1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_5	= (1<<Seg_a)|           (1<<Seg_c)|(1<<Seg_d)|           (1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_6	= (1<<Seg_a)|           (1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
;.equ	Zeichen_7	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|                      (1<<Seg_f)
.equ	Zeichen_8	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_9	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)|           (1<<Seg_f)|(1<<Seg_g)

;alternatives Ziffernbild fr die Ziffern 6, 7 und 9 mit jeweils einem Segment weniger als Obige
;.equ	Zeichen_6	=                       (1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_7	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)
;.equ	Zeichen_9	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|                      (1<<Seg_f)|(1<<Seg_g)

;Buchstaben
.equ	Zeichen_A	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|           (1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_b_kl	=                       (1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_C	= (1<<Seg_a)|                      (1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)
.equ	Zeichen_c_kl	=                                  (1<<Seg_d)|(1<<Seg_e)|           (1<<Seg_g)
.equ	Zeichen_d_kl	=            (1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|           (1<<Seg_g)
.equ	Zeichen_E	= (1<<Seg_a)|                      (1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_F	= (1<<Seg_a)|                                 (1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_H	=            (1<<Seg_b)|(1<<Seg_c)|           (1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_h_kl	=                       (1<<Seg_c)|           (1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_J	=            (1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)
.equ	Zeichen_L	=                                  (1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)
.equ	Zeichen_mikro	=            (1<<Seg_b)|                      (1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_N	= (1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|           (1<<Seg_e)|(1<<Seg_f)
.equ	Zeichen_n_kl	=                       (1<<Seg_c)|           (1<<Seg_e)|           (1<<Seg_g)
.equ	Zeichen_O	= Zeichen_0
.equ	Zeichen_o_kl	=                       (1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|           (1<<Seg_g)
.equ	Zeichen_P	= (1<<Seg_a)|(1<<Seg_b)|                      (1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_r_kl	=                                             (1<<Seg_e)|           (1<<Seg_g)
.equ	Zeichen_S	= Zeichen_5
.equ	Zeichen_t_kl	=                                  (1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)
.equ	Zeichen_U	=            (1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)
.equ	Zeichen_u_kl	=                       (1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)

;Zeichen
.equ	Zeichen_minus	=                                                                   (1<<Seg_g)
.equ	Zeichen_leer	= 0
.equ	Textende	= 0xFF
;Zeichen mit Dezimalpunkt
.equ	Zeichen_Punkt	= (1<<Seg_dp)
.equ	Zeichen_1_dp	=            (1<<Seg_b)|(1<<Seg_c)|                                (1<<Seg_dp)
.equ	Zeichen_S_dp	= (1<<Seg_a)|           (1<<Seg_c)|(1<<Seg_d)|(1<<Seg_f)|(1<<Seg_g)|(1<<Seg_dp)

;*********************************************************************************************************
;***************  Zuweisungen der I/O-Adressen							**********
;*********************************************************************************************************
.equ	scaler1		= tccr1b	;Vorteiler Timer 1
.equ	timer2		= tcnt2		;Timer 2
.equ	scaler2		= tccr2		;Vorteiler Timer 2
.equ	compare2	= ocr2		;Output-Compare-Register

;*********************************************************************************************************
;***************  Registerbelegung      							**********
;***************  Befehle, die nur mit den High-Registern r16 bis r31 mglich sind:		**********
;***************  LDI,ANDI,CBR,CPI,SBCI,SBR,ORI,SER,SUBI					**********
;*********************************************************************************************************

;			= r0		;Ergebnisregister Multiplikation LowByte
;			= r1		;Ergebnisregister Multiplikation HighByte
.def	mod0		= r2		;  } LSByte
.def	mod1		= r3		;  }      40-Bit Rechenregister 
.def	mod2		= r4		;  }		Registerbelegung "mod" nicht ndern, damit Routine
.def	mod3		= r5		;  }		SQRT32 von Ruud v. Gessel ohne nderungen luft
.def	mod4		= r6		;  } MSByte
.def	s		= r7		;zum sichern des Statusregisters innerhalb der Timer1-IRQs
.def	t		= r8		;zum sichern des Statusregisters innerhalb des Capture-IRQs
.def	Null		= r9		;eine Null wird sehr oft zum Rechnen bentigt
.def	Ticker		= r10		;Zhler Ticks, aufwrtszhlend
;.def			= r11		;frei
;.def			= r12		;frei
.def	tBCD0		= r13		;BCD value digits 1 and 0
.def	tBCD1		= r14		;BCD value digits 3 and 2
.def	tBCD2		= r15		;BCD value digit  4

.def	temp		= r16		;Notizblock		}ergibt bei Bedarf
.def	temp1		= r17		;Hilfsregister 1	}   ein bis zu
.def	temp2		= r18		;Hilfsregister 2	}      32-Bit breites
.def	temp3		= r19		;Hilfsregister 3	}          Rechenregister
.def	var20		= r20		;zweites 32-Bit Operandenregister
.def	var21		= r21 		;
.def	var22		= r22		;
.def	var23		= r23		;
.def	Digit		= r24		;Selektor fr die anzuzeigende Stelle des Display
.def	Flags		= r25		;erstes  FLAG-Register
.def	Flags2		= r26		;zweites FLAG-Register
.def	lc		= r27		;Schleifenzhler exclusiv fr Divisionsroutine elm-chan
;			= r28		;frei
;			= r29		;frei
;	zl		= r30		;Zeiger Z
;	zh		= r31		;Zeiger Z

.def	var10		= r16		;alternative Registernamen
.def	var11		= r17  		;damit die Rechenroutine "div32u" von http://elm-chan.org/cc_e.html
.def	var12		= r18		;ohne nderungen luft
.def	var13		= r19		;

.def	Adr		= r20		;Namen exclusiv
.def	A		= r21		;whrend der
.def	B		= r22		;EEPROM berprfung
.def	C		= r23		;genutzt

;*********************************************************************************************************
;Flag1-Register
;BIT    7         6         5         4         3         2         1         0
;                Null-    Taster   Drehzahl   letzter   letzter   Spulen    Wieder 
;     frei      unter-  verriegelt   wert     Zustand   Zustand              gabe
;              drckung              neu      Licht B   Licht A   erkannt   erkannt

.equ	minus		= 7		;bei negativem Ergebnis Umdrehungen zu Lagenzahl
.equ	Nullunterdr	= 6		;Unterdrckung fhrender Nullen
.equ	verriegelt	= 5		;Taster verriegelt
.equ	Drehzahl_neu	= 4		;neuer Wert "Drehzahl" liegt vor
.equ	Licht_B		= 3		;letzter Zustand der Lichtschranke A
.equ	Licht_A		= 2		;letzter Zustand der Lichtschranke B
.equ	Spulen		= 1		;Spulmodus erkannt
.equ	Wiedergabe	= 0		;Wiedergabemodus erkannt

;*********************************************************************************************************
;Flag2-Register
;BIT    7         6         5         4         3         2         1         0
;     Input    Segment    Dicke      Syn-             /       Anzeigemodus        /
;    capture   Laufzeit             chron     frei    /          0 bis 7          /
;     gltig    gltig    gltig                      /       binr codiert       /

.equ	Capture_val	= 7		;ersten Messwert nach Laufwerksstart verwerfen
.equ	T_segm_val	= 6		;Mittelwert Segmentlaufzeit als plausibel erkannt
.equ	Dicke_val	= 5		;Mittelwert Dicke hat definierte nderungsrate unterschritten
.equ	synchron	= 4		;Rechenwerk auf Kassette gelockt, alle Werte gltig
.equ	modusbit3	= 3		;   }
.equ	modusbit2	= 2		;   }
.equ	modusbit1	= 1		;   } Anzeigemodi 0 bis 7 binr gezhlt
.equ	modusbit0	= 0		;   }


;*********************************************************************************************************
;***************  Belegung des SRAM Speicherbereiches						**********
;*********************************************************************************************************

.equ	digit1		= 0x0060	;auszugebendes Leuchtmuster auf Stelle 1
.equ	digit2		= 0x0061	;auszugebendes Leuchtmuster auf Stelle 2 oder 1 } bei
.equ	digit3		= 0x0062	;auszugebendes Leuchtmuster auf Stelle 3 oder 2 } 4-stelliger
.equ	digit4		= 0x0063	;auszugebendes Leuchtmuster auf Stelle 4 oder 3 } LED
.equ	digit5		= 0x0064	;auszugebendes Leuchtmuster auf Stelle 5 oder 4 } Anzeige
.equ	dp_stelle	= 0x0065	;Position der Dezimalpunkteintastung als Bitmuster !

.equ	Lauf_Impuls	= 0x0066	;Zhler abwrtslaufend Herzschlag zur berwachung Laufwerksbewegung
.equ	Stop_Impuls	= 0x0067	;Zhler abwrtslaufend zum Erzeugen eines langen Nullstopp-Impulses
.equ	Tast_Impuls	= 0x0068	;Zhler abwrtslaufend zum Unterscheiden der Tasterkommandos "kurz" und "lang"
.equ	Mess_Impuls	= 0x0069	;Zhler aufwrtslaufend bestimmt den Ablauf der Synchronisierung
.equ	Calc_Impuls	= 0x006A	;Verzgerer fr Anzeige "CALC" damit sie nicht vor dem Spulen aufblitzt
.equ	frame		= 0x006B	;Zhler Frames
.equ	sekunde		= 0x006C	;Zhler Sekunden
.equ	blinker		= 0x006D	;Zhler abwrtslaufend fr Blinken/Crawlen/Faden im Display
.equ	Feld		= 0x006E	;Zhler fr an den Lichtschranke vorbeilaufende Weifelder
.equ	zeichen_index	= 0x006F	;Adresszeiger Zeichenketten

.equ	scratch_0	= 0x0070	;32-Bit Zwischenablage
.equ	scratch_1	= 0x0071	;
.equ	scratch_2	= 0x0072	;
.equ	scratch_3	= 0x0073	;
.equ	memory_0	= 0x0074	;24-Bit Zwischenablage
.equ	memory_1	= 0x0075	;
.equ	memory_2	= 0x0076	;

.equ	Kernradius_0	= 0x0077	;Wickelkernradius in mm * 100.000
.equ	Kernradius_1	= 0x0078	;
.equ	Kernradius_2	= 0x0079	;

.equ	radius_l	= 0x007A	;Wickelradius
.equ	radius_h	= 0x007B	;
.equ	umdreh_l	= 0x007C	;Anzahl Wickeltellerumdrehungen, rcksetzbar
.equ	umdreh_h	= 0x007D	;
.equ	lagen_l		= 0x007E	;Bandlagenzahl
.equ	lagen_h		= 0x007F	;
.equ	T_Segm_l	= 0x0080	;Winkelgeschwindigkeit Wickelteller
.equ	T_Segm_h	= 0x0081	;
.equ	T_Segm_delta_l	= 0x0082	;alte Winkelgeschwindigkeit Wickelteller
.equ	T_Segm_delta_h	= 0x0083	;
.equ	dicke_f		= 0x0084	;Nachkommastellen Dickenwert (Fraction)
.equ	dicke_l		= 0x0085	;Dickenwert in 0,01m
.equ	dicke_h		= 0x0086	;
.equ	dicke_alt_l	= 0x0087	;Banddicke N+x Sekunden
.equ	dicke_alt_h	= 0x0088	;
.equ	Zeit_calc_l	= 0x0089	;errechnete abgespielte Zeit in Sekunden
.equ	Zeit_calc_h	= 0x008A	;
.equ	Zeit_anz_l	= 0x008B	;quarzgenaue Zeit in Sekunden
.equ	Zeit_anz_h	= 0x008C	;
.equ	Stunden_0	= 0x008D	;Kopfstunden "Wiedergabe"
.equ	Stunden_1	= 0x008E	;1 bit = "Messintervall" Sekunden
.equ	Stunden_2	= 0x008F	;
.equ	hour_l		= 0x0090	;Betriebsstundenanzeige in Stunden
.equ	hour_h		= 0x0091	;
.equ	icr_alt_l	= 0x0092	;Zeitstempel N-1
.equ	icr_alt_h	= 0x0093	;
.equ	dicke_neu_l	= 0x0094	;
.equ	dicke_neu_h	= 0x0095	;
.equ	debug_l		= 0x0096	;zum Debuggen
.equ	debug_h		= 0x0097	;

;ab hier Speicherbereich zur gleitenden Mittelwertbildung V
.equ	Ablage_l	= Speicher-2	;Wert wird umgehend in den Speicher V runtergereicht
.equ	Ablage_h	= Speicher-1	;
.equ	Speicher	= 0x00A0	;Speicherbereich V
.equ	Speicher_V	= Speicher	;Speicher zur Mittelung Drehzahl Aufwickelteller, zur Zeit 192*16 Bit

;*********************************************************************************************************
;***************  Sprungadressen fr die Interrupts						**********
;*********************************************************************************************************
.org	0x0000
	rjmp reset			;Reset
	reti				;External IRQ 0
	reti				;External IRQ 1
	rjmp t2ocm			;Timer 2 Compare 
	rjmp t2ovf			;Timer 2 Overflow
	rjmp t1cev			;Timer 1 Capture Event
	reti				;Timer 1 Compare A
	reti				;Timer 1 Compare B
	reti				;Timer 1 Overflow
	reti				;Timer 0 Overflow
	reti				;SPI Transfer Complete
	reti				;UART Rx Complete
	reti				;UART Data Register Empty
	reti				;UART Tx Compplete
	reti				;ADC Conversion Complete
	reti				;EEPROM Ready
	reti				;Analog Comparator
	reti				;TWI
	reti				;SPM ready	

;*********************************************************************************************************	
reset:
	ldi	temp,high(RAMEND)	;Position Stackpointer festlegen
	out	SPH,temp		;
	ldi	temp,low(RAMEND)	;
	out	SPL,temp		;
;*********************************************************************************************************
;***************  EEPROM initialisieren, falls noch nicht geschehen				**********
	clr	null			;
	rcall	teste_eeprom		;EEPROM auf Korruption durchsuchen und reparieren
	ldi	temp1,1			;wenn die Zelle 1
	rcall	eeread			;
	ldi	temp1,0b10110100	;dieses Bitmuster enthlt
	cpse	temp,temp1		;ist das EEPROM schon initialisiert
	rcall	init_eeprom		;sonst, EEPROM initialisieren
;*********************************************************************************************************
;****************  gerettete Werte holen							**********
	rcall	loesche_sram		;mal nass durchwischen und dann

	ldi	temp1,adr_eeprom+0	;hole gerettete Daten aus dem EEPROM
	rcall	eeread			;
	sts	stunden_0,temp		;
	ldi	temp1,adr_eeprom+1	;
	rcall	eeread			;
	sts	stunden_1,temp		;
	ldi	temp1,adr_eeprom+2	;
	rcall	eeread			;
	sts	stunden_2,temp		;die Betriebsstunden

	ldi	temp1,Adr_eeprom+3	;
	rcall	eeread			;
	sts	umdreh_l,temp		;
	ldi	temp1,adr_eeprom+4	;
	rcall	eeread			;
	sts	umdreh_h,temp		;den letzten Zhlwerksstand

	ldi	temp1,adr_eeprom+5	;
	rcall	eeread			;
	sts	dicke_l,temp		;
	ldi	temp1,adr_eeprom+6	;
	rcall	eeread			;
	sts	dicke_h,temp		;den letzten Dickenwert
;*********************************************************************************************************
;***************  Initialisierung 		 						**********
;*********************************************************************************************************
;***************  nicht bentigte Resourcen abschalten  **************************************************
	clr	temp			;
	out	ADCSRA,temp		;AD-Wandler abschalten
	ldi	temp,(1<<ACD)		;
	out	ACSR,temp		;analog Komparator abschalten
	ldi	temp,(1<<WDCE)|(1<<WDE)	;
	out	WDTCR,temp		;Watchdog-Timer vorbereiten
	ldi	temp,(1<<WDCE)		;
	out	WDTCR,temp		;Watchdog-Timer abschalten
;***************  Timer, Vorteiler und Timer-IRQs konfigurieren  *****************************************
	ldi	temp,(1<<CS21)		;Prescaler Timer2 auf CK/8 einstellen
	out	scaler2,temp		;
	ldi	temp,(1<<TICIE1)|(1<<TOIE2)|(1<<OCIE2)
	out	timsk,temp		;Input1Capture-, Timer2Overflow- und Compare2Match IRQ freigeben
	ldi	temp,(1<<ICNC1)|(1<<ICES1)|(1<<CS11)|(1<<CS10);Prescaler Timer1 auf CK/64
	out	scaler1,temp		;Capture-Event auf steigende Flanke mit NoiseCanceler
;***************  Portpins definieren  *******************************************************************
	ldi	temp,(1<<a_stop)	;
	out	ddrb,temp		;alle Pins PortB Eingang auer Nullstopausgang
	sbi	portb,e_taster		;PullUp Tasteneingang setzen
.if (anzeigestellen==4)			;nur bei 4 Digits Anzeigeumfang
	ldi	temp,(1<<a_digit2)|(1<<a_digit3)|(1<<a_digit4)|(1<<a_digit5)|(1<<a_ir_led)
.else					;nur bei 5 Digits Anzeigeumfang
	ldi	temp,(1<<a_digit1)|(1<<a_digit2)|(1<<a_digit3)|(1<<a_digit4)|(1<<a_digit5)|(1<<a_ir_led)
.endif
	out	ddrc,temp		;setzen der Portpins C als Ausgang
	ldi	temp,(1<<Seg_a)|(1<<Seg_b)|(1<<Seg_c)|(1<<Seg_d)|(1<<Seg_e)|(1<<Seg_f)|(1<<Seg_g)|(1<<Seg_dp)
	out	ddrd,temp		;setzen der Portpins D als Ausgang zur Steuerung der Kathoden
;***************  Register vorbereiten  ******************************************************************
	cbi	portc,a_ir_led		;IR-LEDs einschalten
	ldi	temp,((helligkeit*250)/100)+5
	out	compare2,temp		;stelle Sollhelligkeit ein
	ldi	temp,frameoffset	;
	sts	frame,temp		;
	ldi	digit,1			;Initialisierung Stellenselektor
	ldi	flags2,Startmodus	;zeige nach dem Einschalten gewnschten Modus
	ldi	temp,kennung		;lade Anzeigedauer
	sts	blinker,temp		;fr Kennbuchstaben
;*********************************************************************************************************
	sbis	pinb,e_taster		;wenn Taster beim Einschalten gedrckt ist
	rcall	irgendwas		;mache irgendwas sinnvolles
;*********************************************************************************************************
	rcall	init_kernradius		;Wickelkernradius auf Defaultwert setzen
	rcall	zaehle_radius		;Radius auf Minimum setzen
	lds	temp,dicke_l		;letzter verwendeter Dickenwert
	lds	temp1,dicke_h		;
	rcall	init_Speicher_U		;in Mittelwertspeicher U geben
	rcall	rechne_betriebsstunden	;einmal Betriebsstunden rechnen und bereitstellen
;***************  Der Drehgeber muss nach dem Einschalten ein "0" ausgeben, sonst gibts Murks  ***********
	in	flags,pinb		;sichere Eingangsstatus
	andi	flags,(1<<e_geber_b)|(1<<e_geber_a);maskiere Nutzdaten der Lichtschranken aus
	lsl	flags			;
	lsl	flags			;neuer Zustand wird alter Zustand
;***************  Ende  Initialisierung  *****************************************************************
	sei				;Interrupts freigeben

;*********************************************************************************************************
;***************  Programmstart									**********
;*********************************************************************************************************
start:
	rcall	hole_drehgeber		;Drehgeber pollen
	sbis	pinb,e_spannung		;wenn Eingangsspannung abnimmt
	rcall	rette_werte		;Display abschalten und wichtige Werte ins EEPROM retten
;***************  neuen Messwert "Drehzahl" in die Mittelwertbildung bergeben  **************************
	sbrs	flags,Drehzahl_neu	;wenn ein neuer gltiger Messwert vorliegt
	rjmp	ende_neuer_wert		;
	rcall	in_speicher_v		;den Messwert in den Mittelwertspeicher bernehmen
	cbr	flags,(1<<Drehzahl_neu)	;erledigt
ende_neuer_wert:
;***************  Ticker wird vom Timerinterrupt mit 2000 Hz hochgezhlt  ***********************
	mov	temp,ticker		;sichere Ticker
	cpi	temp,80			;prfe ob 1 Frame um
	brlo	start			;wenn nicht, Zeitzhlung verlassen
	clr	ticker			;Ticker rcksetzen
;***************  Durchlauf hier jedes Frame  ************************************************************
;***************  Behandlung Dauer des Nullstopp-Impulses  ***********************************************
	lds	temp,stop_impuls	;hole Impulszeitenzhler
	tst	temp			;wenn Nullstoppimpuls andauert
	breq	kein_null_impuls	;
	dec	temp			;warte noch ein Frame
	sts	stop_impuls,temp	;
	rjmp	ende_nullstop		;
kein_null_impuls:
	cbi	portb,a_stop		;sonst Nullstopp-Ausgang deaktivieren
ende_nullstop:
;***************  Behandlung Blinker Display  ************************************************************
	lds	temp,blinker		;
	tst	temp			;den Zeichenfolgezhler
	breq	ende_blinken		;
	dec	temp			;bis auf Null runterzhlen
	sts	blinker,temp		;
ende_blinken:
;***************  Behandlung Taster  *********************************************************************
	lds	temp,tast_impuls	;
	sbic	pinb,e_taster		;wenn Taster gedrckt ist
	rjmp	teste_auf_kurz		;
	inc	temp			;Zhler zum Messen der Dauer des Tastendrucks bedienen
	sts	tast_impuls,temp	;
	cpi	temp,25			;Taste schon 1 Sekunde lang gedrckt?
	brlo	ende_taster		;
	rjmp	Kommando_lang		;dann Tasterkommando "lang" ausfhren
teste_auf_kurz:
	cpi	temp,25			;
	brsh	teste_auf_entriegeln	;
	cpi	temp,3			;mgliche Prellzeit um?
	brlo	ende_taster		;nein, raus
	sbrs	flags,verriegelt	;wenn entriegelt ist
	rjmp	kommando_kurz		;Tasterkommando "kurz" ausfhren
teste_auf_entriegeln:
	inc	temp			;
	sts	tast_impuls,temp	;
	cpi	temp,25+5		;
	breq	entriegeln		;
	rjmp	ende_taster		;
entriegeln:
	clr	temp			;
	sts	tast_impuls,temp	;
	cbr	flags,(1<<verriegelt)	;
	rjmp	ende_taster		;
;***************  Tastenkommando "kurz" schaltet zwischen den Anzeigemodi rund  **************************
kommando_kurz:
	ldi	temp,kennung		;lade Anzeigedauer
	sts	blinker,temp		;fr den Kennbuchstaben
	mov	temp,flags2		;sichere Modusflags
	andi	temp,(1<<modusbit3)|(1<<modusbit2)|(1<<modusbit1)|(1<<modusbit0);Nutzdaten maskieren
	cpi	temp,modi+startmodus-1	;nach dem letzten Modus kommt
	brsh	setze_modus_0		;wieder Modus 0
	inc	flags2			;nchster Anzeigemodus bitte
	inc	temp			;
	cpi	temp,4			;im Textmodus
	brne	kommando_kurz_erledigt	;
	clr	temp			;
	sts	blinker,temp		;keine Pause fr Kennbuchstaben und
	sts	zeichen_index,temp	;Zeichenkette fngt beim Aufruf immer VORNE an
	rjmp	kommando_kurz_erledigt	;
setze_modus_0:
	cbr	flags2,(1<<modusbit3)|(1<<modusbit2)|(1<<modusbit1)|(1<<modusbit0)
kommando_kurz_erledigt:
	clr	temp			;
	sts	tast_impuls,temp	;Zhler rcksetzen
	rjmp	ende_taster		;
;***************  Tastenkommando "lang" stellt das Zhlwerk auf 0000  ************************************
kommando_lang:
	clr	temp			;
	sts	umdreh_l,temp		;Umdrehungszhler
	sts	umdreh_h,temp		;und
	sts	feld,temp		;Felderzhler nullen

;cbr	flags2,(1<<T_segm_val)|(1<<Dicke_val)|(1<<synchron)

	sbr	flags,(1<<verriegelt)	;Kurzkommando verriegeln
	ldi	temp,25			;
	sts	tast_impuls,temp	;Zhler setzen
ende_taster:
;***************  Laufwerksberwachung  ******************************************************************
	sbic	pinb,e_tape_out		;wenn das Kassettenfach offen ist
	rjmp	kassettenfach_zu	;
	cbr	flags2,(1<<t_segm_val)|(1<<dicke_val)|(1<<synchron)
	clr	temp			;endet die Synchronitt und
	sts	lagen_l,temp		;und einige Anzeigewerte werden hier mal
	sts	lagen_h,temp		;GENULLT
	sts	zeit_calc_l,temp	;
	sts	zeit_calc_h,temp	;
	sts	zeit_anz_l,temp		;
	sts	zeit_anz_h,temp		;
	rcall	init_kernradius		;Wickelkernradius auf Defaultwert setzen
	rcall	zaehle_radius		;Radius auf Minimum setzen
kassettenfach_zu:
;*********************************************************************************************************
;***************  In welchem Zustand ist das Bandlaufwerk gerade?
;***************  Stopp, Spulen oder Wiedergabe
;*********************************************************************************************************
	lds	temp1,lauf_impuls	;hole Laufwerksherzschlag
	tst	temp1			;ohne "Herzschlag" ist das Laufwerk gestoppt,
	breq	laufwerk_steht		;bediene die Flags und Lagenanzahl
	dec	temp1			;bei vorhandenem "Herzschlag"
	sts	lauf_impuls,temp1	;
	mov	temp,flags		;sichere Flags
	andi	temp,(1<<wiedergabe)|(1<<spulen);maskiere Laufwerksbetriebsart aus
	cpi	temp,(1<<wiedergabe)	;wenn ein Frame vorher Wiedergabe detektiert wurde
	breq	laufwerk_spielt		;bitteschn, Flags bedienen
	cpi	temp1,laufwerkpuls-1	;ist jetzt Wiedergabegeschwindigkeit vorhanden?
	brlo	laufwerk_spielt		;ja (weil der "Herzschlag" runtergezhlt werden kann)
	sbrc	flags,spulen		;nein, der "Herzschlag" bleibt immer auf Maximum
	rjmp	laufwerk_spult		;das geht nur im Spulmodus und wurde ein Frame vorher schon ermittelt
	sbr	flags,(1<<spulen)	;beim ersten mal Detektieren hier Flag bedienen
	rjmp	ende_laufwerk		;
;*********************************************************************************************************
;***************  im Spulmodus wird der Lagenzhler mitgefhrt (nicht gerechnet)
;***************  und daraus die abgelaufene Zeit die an die Anzeige bergeben wird.
;*********************************************************************************************************
laufwerk_spult:
	cbr	flags,(1<<wiedergabe)	;Flag bedienen
	rcall	sekunden_absolut	;errechne die abgespielten Sekunden aus Lagen und Dicke
	rcall	uebergebe_zeit		;errechnete Zeit sofort komplett an die Anzeige bergeben
	rcall	zaehle_radius		;Radius aus Banddicke und Bandlagen ermitteln
	clr	temp			;bei spulendem Laufwerk findet kein Messzyklus statt
	sts	mess_impuls,temp	;
	rjmp	ende_laufwerk		;und raus
laufwerk_spielt:
	sbr	flags,(1<<wiedergabe)	;Flags bedienen
	cbr	flags,(1<<spulen)	;
	rjmp	ende_laufwerk		;und raus
laufwerk_steht:
	cbr	flags,(1<<wiedergabe)|(1<<spulen);Flags bedienen
	cbr	flags2,(1<<capture_val)|(1<<t_segm_val)
	rcall	setze_lagenzahl		;und Lagenzahl auf den zur Zeitanzeige passenden Wert setzen
	clr	temp			;bei stehendem Laufwerk findet kein Messzyklus statt
	sts	mess_impuls,temp	;
ende_laufwerk:
;***************  Behandlung neues Displaybild  **********************************************************
.if (MessAnzeige==1)
	sbrc	flags2,synchron		;bei Sync-Flag AUS
	rjmp	zeige_nicht_calc	;UND
	sbrc	flags,wiedergabe	;bei Wiedergabe
	rjmp	Berechnung_laeuft	;"Berechnung" anzeigen
zeige_nicht_calc:
	clr	temp			;
	sts	blinker,temp		;
.endif
	ldi	temp,((helligkeit*250)/100)+5
	out	compare2,temp		;stelle Sollhelligkeit ein

	sbis	pinb,e_tape_out		;wenn das Kassettenfach offen ist
	rjmp	kassette_offen		;zeig es an

	mov	temp,flags2		;sichere Flags2
	andi	temp,(1<<modusbit3)|(1<<modusbit2)|(1<<modusbit1)|(1<<modusbit0);Nutzdaten Modus maskieren
	cpi	temp,15			;
	breq	modus15			;und in den entsprechenden Modus verzweigen
	cpi	temp,14			;
	breq	modus14			;
	cpi	temp,13			;
	breq	modus13			;
	cpi	temp,12			;
	breq	modus12			;
	cpi	temp,11			;
	breq	modus11			;
	cpi	temp,10			;
	breq	modus10			;
	cpi	temp,9			;
	breq	modus9			;
	cpi	temp,8			;
	breq	modus8			;
	cpi	temp,7			;
	breq	modus7			;
	cpi	temp,6			;
	breq	modus6			;
	cpi	temp,5			;
	breq	modus5			;
	cpi	temp,4			;
	breq	modus4			;
	cpi	temp,3			;
	breq	modus3			;
	cpi	temp,2			;
	breq	modus2			;
	cpi	temp,1			;
	breq	modus1			;
	cpi	temp,0			;
	breq	modus0			;
Modus15:
	rcall	zeige_debug		;"d WERT"
	rjmp	sync_zeichen		;
Modus14:
	rcall	zeige_radius		;"r WICKELRADIUS" errechnet
	rjmp	sync_zeichen		;
Modus13:
	rcall	zeige_drehzahl_delta	;"J SEGMENTLAUFZEIT" Umdrehung N-4 bis N-8
	rjmp	kein_sync_zeichen	;
Modus12:
	rcall	zeige_drehzahl		;"n SEGMENTLAUFZEIT" Umdrehung N bis N-3
	rjmp	sync_zeichen		;
Modus11:
	rcall	zeige_segmentlaufzeit	;"P TIMERWERT" des letzten Segmentvorbeilaufs
	rjmp	sync_zeichen		;
Modus10:
	rcall	zeige_abw_sekunden	;"F FEHLER" Sekundenrechner zu Zeitanzeige
	rjmp	sync_zeichen		;
Modus9:
	rcall	zeige_sekunden		;"S SEKUNDEN" errechnet
	rjmp	sync_zeichen		;
Modus8:
	rcall	zeige_dicke_4		;" BANDDICKE" 4-stellig mit Hundertstel
	rjmp	sync_zeichen		;
Modus7:
	rcall	zeige_banddicke		;"b BANDDICKE" einlaufender ungefilterter Wert
	rjmp	sync_zeichen		;
Modus6:
	rcall	zeige_abw_lagen		;"A ABWEICHUNG" Lagenzhler zu Umdrehungszhler
	rjmp	sync_zeichen		;
Modus5:
	rcall	zeige_bandlagen		;"L BANDLAGEN"
	rjmp	sync_zeichen		;
Modus4:
	rcall	zeige_text		;"LAUFTEXT" max. 254 Zeichen
	rjmp	kein_sync_zeichen	;
Modus3:
	rcall	zeige_kopfstunden	;"h KOPFSTUNDEN"
	rjmp	sync_zeichen		;
Modus2:
	rcall	zeige_dicke		;" BANDDICKE"
	rjmp	sync_zeichen		;
Modus1:
	rcall	zeige_umdrehungen	;"U UMDREHUNGEN" gezhlt, rcksetzbar
	rjmp	sync_zeichen		;
Modus0:
	rcall	zeige_zeit		;"t STUNDEN:MINUTEN:SEKUNDEN" quarzgesteuert
	rjmp	sync_zeichen		;
kassette_offen:
.if (anzeigestellen==4)			;nur bei 4 Digits
	rcall	zeige_open		;"OPEN"
.else					;nur bei 5 Digits
	rcall	zeige_offen		;"OFFEN"
.endif
	rjmp	kein_sync_zeichen	;
;***************  L A G E R  ***************************************************************
;	rcall	zeige_leer		;"-.--.--"
;	rcall	zeige_fast_leer		;"---.--"
;	rcall	zeige_ganz_leer		;"-----"
;	rcall	zeige_calc_bli		;"CALC" blinkend
;***************  L A G E R  ***************************************************************
Berechnung_laeuft:
	rcall	zeige_calc_dim		;"CALC" auf- und abschwellend
.if (MessAnzeige==1)
	clr	temp			;
	sts	tast_impuls,temp	;Tasterkommandos solange sperren
.endif
sync_zeichen:
;	lds	temp,digit4		;
;	sbrc	flags2,capture_val	;wenn Capture freigegeben ist
;	sbr	temp,(1<<seg_dp)	;Dezimalpunkt auf Digit 4 zuschalten
;	sts	digit4,temp		;
.if (MessAnzeige==0)
	lds	temp,digit5		;
	sbrc	flags2,synchron		;wenn synchron
	sbr	temp,(1<<seg_dp)	;Dezimalpunkt auf Digit 5 zuschalten
	sts	digit5,temp		;
.endif
kein_sync_zeichen:
ende_displaybild:
;*********************************************************************************************************
	lds	temp,frame		;hole Frames
	inc	temp			;Frame erhhen
	sts	frame,temp		;Frames sichern
	cpi	temp,25+frameoffset	;prfe ob 25 Frames vorbei
	brlo	ende_zeit		;
	ldi	temp,frameoffset	;Frames zurcksetzen
	sts	frame,temp		;
;*********************************************************************************************************
;***************  Durchlauf hier jede Sekunde
;*********************************************************************************************************
;im Playmodus kann ich die Drehzahl ermitteln und daraus Dickenwert, Lagenzahl und abgelaufene Zeit errechnen
;bei gltiger Wiedergabegeschwindigkeit wird der Dickenwert errechnet (Flag: T-Segm-val gesetzt) und im
;Filter U beruhigt.
;Sobald sich ein stabiler Wert Dicke eingestellt hat (Flag: dicke-val gesetzt) werden daraus die
;absoluten Lagen und absoluten Sekunden errechnet und einmal an das Uhrwerk bergeben.(Flag: synchron gesetzt)
;Die Filtertife U wird jetzt auf 512 Werte angehoben
;Ab jetzt kann die Quarzzeit nur noch sukzessive an im die Hintergrund weiterhin errechnete Zeit nachgefhrt werden.
;Ein Auswurf der Kassette oder ein Reset lschen alle Synchronflags.
;*********************************************************************************************************
	mov	temp,flags		;sichere Flags
	andi	temp,(1<<wiedergabe)|(1<<spulen);maskiere Laufwerksbetriebsart aus
	cpi	temp,(1<<wiedergabe)	;wenn Wiedergabe detektiert wurde und
	brne	ende_ablaufsteuerung	;
	sbrs	flags2,dicke_val	;bei eingependeltem Dickenwert die
	rjmp	quarzzeit_bitte		;
	rcall	lagen_absolut		;absoluten Bandwickellagen aus Tellerdrehzahl und Dickenwert errechnen
	rcall	sekunden_absolut	;und die abgespielten Sekunden aus Lagenzahl und Dickenwert errechnen
	sbrc	flags2,synchron		;
	rjmp	quarzzeit_bitte		;
	rcall	uebergebe_zeit		;die errechnete Zeit einmal (bis zum nchsten Reset) komplett bergeben
	sbr	flags2,(1<<synchron)	;erledigt
quarzzeit_bitte:
	rcall	synchronisiere_zeit	;generiere Quarzzeit, nach Sync.fhre sie an errechnete Zeit heran
ende_ablaufsteuerung:
;*********************************************************************************************************
	sbrs	flags,synchron		;nach erfolgter Synchronisirung
	rcall	validiere_u		;ist keine Validierung der Banddicke mehr ntig
;*********************************************************************************************************
	lds	temp,sekunde		;
	inc	temp			;Sekunde erhhen
	sts	sekunde,temp		;
	cpi	temp,kopfintervall	;Betriebsstundenintervall erreicht
	brlo	ende_zeit		;
;*********************************************************************************************************
;***************  Durchlauf hier alle Kopfintervall = 10 Sekunden				**********
;*********************************************************************************************************
;***************  Betriebsstunden zhlen  ****************************************************************
	sbrs	flags,wiedergabe	;nur bei Wiedergabe
	rjmp	ende_betriebsstunden	;
	rcall	addiere_Kopfzeit	;Koptstunden zhlen
	rcall	rechne_betriebsstunden	;und einmal in Stunden umrechnen
ende_betriebsstunden:
	clr	temp			;
	sts	sekunde,null		;Sekunden rcksetzen
;*********************************************************************************************************
ende_zeit:
	rjmp	start			;Ende Hauptschleife

;*********************************************************************************************************
;***************  Interruptroutine zum Auffrischen des Displays, 2KHz Aufruffrequenz		**********
;***************  benutzte Register: temp[1:0], digit, Z, flags, ticker, timer0, dp_stelle	**********
;***************                     portc, portd						**********
;*********************************************************************************************************
t2ovf:
	in	s,sreg			;Statusregister
	push	temp			;sichere Register temp
	push	temp1			;sichere Register temp1
	push	zl			;
	push	zh			;
	inc	ticker			;Ticker bedienen
	ldi	temp,9			;Timer vorladen fr
	out	timer2,temp		;genauen 0,5ms-berlauf
;***************  Ab hier Ansteuerung des Display  *******************************************************
;***************  Anzeigestelle rotieren  ****************************************************************
	lsl	digit			;nchste Stelle ansteuern
	cpi	digit,((1<<a_digit5)+1)	;nach dem letzten Digit kommt
	brlo	Stelle_gueltig		;
.if (anzeigestellen==4)			;nur bei 4 Digits
	ldi	digit,(1<<a_digit2)	;wieder das erste
.else					;nur bei 5 Digits
	ldi	digit,(1<<a_digit1)	;wieder das erste
.endif
Stelle_gueltig:
	cpi	digit,(1<<a_digit2)	;Unterdrckung fhrender Nullen erst ab Digit 2 aktiv
	brne	kein_digit_2		;
	mov	temp,flags2		;
	andi	temp,7
	cpi	temp,0			;nie im Modus0 "Zeitanzeige"
	breq	kein_nullunterd_modus	;
	cpi	temp,7			;nie im Modus7 "Textmeldung"
	sbic	pinb,e_tape_out		;nie wenn das Kassettenfach offen ist
	sbr	flags,(1<<nullunterdr)	;
kein_nullunterd_modus:
kein_digit_2:
	in	temp,portc		;sichere Portstatus
.if (Anzeigeart==1)			;fr Anzeigen mit gemeinsamer Anode
	ori	temp,~(1<<A_IR_LED)	;maskiere IR-LED-Ausgang aus
	mov	temp1,digit		;sichere Digit
	com	temp1			;invertiere
	and	temp,temp1		;Digit dazunehmen
.else
	andi	temp,(1<<A_IR_LED)	;maskiere IR-LED-Ausgang aus und
	or	temp,digit		;Digit dazunehmen
.endif
	out	portc,temp		;aktuelle Anzeigestelle einschalten
	sbrc	digit,a_digit5		;eine "0" auf dem letzten Digit
	cbr	flags,(1<<nullunterdr)	;wird immer angezeigt
;***************  Leuchtmuster fr diese Anzeigestelle holen  ********************************************
	rcall	stellenselektor		;hole Index der Anzeigestelle
	clr	zh			;
	ldi	zl,(digit1-1)		;Zeiger einstellen
	add	zl,temp			;Zeiger genau auf dazugehrige SRAM-Zelle drehen
	ld	temp,z			;Leuchtbild holen
;***************  Behandlung Dezimalpunkt  ***************************************************************
	lds	temp1,dp_stelle		;
	and	temp1,digit		;maskieren, damit ich mehrere Dezimalpunkte gleichzeitig eintasten kann
	cp	digit,temp1		;wenn kein Dezimalpunkt an dieser Anzeigestelle gewnscht
	brne	ende_dezimalpunkt	;raus, sonst
	ori	temp,zeichen_punkt	;Dezimalpunkt dazuschalten
ende_dezimalpunkt:
.if (nullunterdrueck)
;***************  Unterdrckung fhrender Nullen im Display  *********************************************
	sbrs	flags,nullunterdr	;bei aktiver Nullunterdrckung
	rjmp	ende_null		;
	cpi	temp,(1<<seg_a)|(1<<seg_b)|(1<<seg_c)|(1<<seg_d)|(1<<seg_e)|(1<<seg_f)
	breq	ist_null		;auf 0 ohne Dezimalpunkt testen, denn die wird nicht gezeigt
	cbr	flags,(1<<nullunterdr)	;andere Zeichen beenden die Nullunterdrckung
	rjmp	ende_null		;
ist_null:
	clr	temp			;
ende_null:
.endif
;***************  Ausgabe des Zeichens auf dem Display  **************************************************
.if (Anzeigeart==1)			;fr Anzeigen mit gemeinsamer Anode
	com	temp			;Signal invertieren
.endif
	out	portd,temp		;Zeichen raus auf die Anzeige
;***************  Ende Interupt  *************************************************************************
	pop 	zh			;
	pop	zl			;
	pop	temp1			;Register temp1 wieder herstellen
	pop	temp			;Register temp wieder herstellen
	out	sreg,s			;Statusregister wieder herstellen
	reti				;und zurck in die Hauptroutine

;*********************************************************************************************************
;***************  Interruptroutine zum Abschalten der Anzeige					**********
;***************  Hiermit wird die Displayhelligkeit eingestellt				**********
;***************  benutzte Register: temp, portc, portd						**********
;*********************************************************************************************************
t2ocm:
	in	s,sreg			;Statusregister sichern
	push	temp			;Register temp sichern
	rcall	anzeige_aus		;Anzeige dunkelschalten
	pop	temp			;Register temp wiederherstellen
	out	sreg,s			;Statusregister wiederherstellen
	reti				;

;*********************************************************************************************************
;***************  Interruptroutine zur Messung der Zeit zwischen 2 Weifeldflanken		**********
;***************  benutzte Register: s, temp[1:0], ICR1, TCNT1, ablage				**********
;*********************************************************************************************************
t1cev:
	sei				;damit der Anzeigemultiplex jederzeit dazwischen kann
	in	t,sreg			;Statusregister sichern
	push	temp			;Arbeitsregister sichern
	push	temp1			;
	push	temp2			;
	push	temp3			;
	in	temp,icr1l		;sichere Zeitstempel
	in	temp1,icr1h		;
	lds	temp2,icr_alt_l		;hole letzten Zeitstempel
	lds	temp3,icr_alt_h		;
	sts	icr_alt_l,temp		;neuer Zeitstempel wird Alter
	sts	icr_alt_h,temp1		;
	sbrs	flags2,capture_val	;wenn das Laufwerk stand oder spulte
	rjmp	ende_capture_event	;ist der erste Messwert nicht gltig
	sub	temp,temp2		;wieviel Counts sind seit letztem Zeitstempel vergangen?
	sbc	temp1,temp3		;ein bertrag muss nicht bercksichtigt werden
;***************  Messwert "Drehzahl" validieren  ********************************************************
	cpi	temp,byte1(max_play)	;
	ldi	temp2,byte2(max_play)	;
	cpc	temp1,temp2		;wenn der Impuls die Mindestdauer fr Wiedergabebetrieb hatte
	brlo	ende_capture_event	;
	sts	ablage_l,temp		;merke sie dir
	sts	ablage_h,temp1		;und signalisiere das
	sbr	flags,(1<<Drehzahl_neu)	;
ende_capture_event:
	sbr	flags2,(1<<capture_val)	;
	pop	temp3			;
	pop	temp2			;
	pop	temp1			;
	pop	temp			;Arbeitsregister wiederherstellen
	out	sreg,t			;Statusregister wiederherstellen
	reti				;

.include"unterprogramme.inc"
.include"anzeigeroutinen.inc"
.include"arithmetikroutinen.inc"
.include"daten.inc"
